@@ -63,6 +63,9 @@
 #include <glib.h>
 
 #include "hildon-file-common-private.h"
+
+/*#define DEBUG*/
+
 #define HILDON_RESPONSE_FOLDER_BUTTON 12345
 #define HILDON_RESPONSE_FOLDER_CREATED 54321
 /* Common height for filetrees. About 8 lines. Filetree sets default margins,
@@ -73,6 +76,12 @@
 #define FILE_SELECTION_WIDTH_TOTAL 590  /* Width for full filetree (both
                                            content and navigation pane) */
 
+#ifdef DEBUG
+#define DBG(args...) fprintf (stderr, ## args)
+#else
+#define DBG(...) do { } while (0)
+#endif
+
 static void sync_extensions_combo (HildonFileChooserDialogPrivate *priv);
 
 #define HILDON_FILE_CHOOSER_DIALOG_TYPE_SELECTION_MODE (hildon_file_chooser_dialog_selection_mode_get_type())
@@ -117,8 +126,8 @@ struct _HildonFileChooserDialogPrivate {
     GtkSizeGroup *caption_size_group;
     GtkWidget *caption_control_name;
     GtkWidget *caption_control_location;
-    GtkWidget *label_location;
     GtkWidget *entry_name;
+    GtkWidget *eventbox_location;
     GtkWidget *hbox_location, *image_location, *title_location;
     GtkWidget *extensions_combo;
     GtkFileChooserAction action;
@@ -277,8 +286,7 @@ get_path_length_from_uri(const char *uri)
   if (!unescaped) return 0;
 
   ULOG_DEBUG("Original uri = %s", uri);
-  /* Should Not be utf8, but G_FILENAME_ENCODING, BUT it still is. */
-  len = g_utf8_strlen(unescaped, -1);
+  len = strlen (unescaped);
   ULOG_DEBUG("Unescaped path = %s, length = %d", unescaped, len);
   g_free(unescaped);
 
@@ -356,11 +364,21 @@ hildon_file_chooser_dialog_save_multiple_set(HildonFileChooserDialogPrivate *pri
   return text && strlen(text) > 0;
 }
 
-static void
+/* 
+   Sets content to name entry. 
+   Returns TRUE if content has changed, FALSE if not 
+*/ 
+static gboolean
 set_entry (GtkWidget *entry, const char *name, const char *ext)
 {
   if (name == NULL)
-    return;
+    return FALSE;
+
+  gchar *orig_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+  const gchar *new_name;
+  gboolean retval;
+
+  DBG ("SET ENTRY '%s' '%s'\n", name, ext);
 
   gtk_entry_set_text (GTK_ENTRY(entry), name);
 
@@ -375,6 +393,17 @@ set_entry (GtkWidget *entry, const char *name, const char *ext)
                                 ext, strlen (ext),
                                 &position);
     }
+
+  new_name = gtk_entry_get_text (GTK_ENTRY (entry));
+
+  if (strcmp (new_name, orig_name) == 0)
+	retval = FALSE;
+  else
+    retval = TRUE;
+
+  g_free (orig_name);
+  return retval;
+
 }
 
 static gchar *
@@ -406,52 +435,50 @@ hildon_file_chooser_dialog_do_autonaming(HildonFileChooserDialogPrivate *
                                          priv)
 {
     g_assert(HILDON_IS_FILE_SELECTION(priv->filetree));
+	
+	gboolean changed;
 
     if (GTK_WIDGET_VISIBLE(priv->caption_control_name) &&
-        priv->stub_name && !priv->edited)
+        priv->stub_name && priv->stub_name[0] && !priv->edited)
     {
         gchar *name = NULL;
-        gboolean selection;
-        gint pos;
 
         g_signal_handler_block(priv->entry_name, priv->changed_handler);
         if (priv->autonaming_enabled) {
             GtkTreeIter iter;
 
-            ULOG_INFO("Trying [%s] [%s]", priv->stub_name, priv->ext_name);
+            DBG ("Trying [%s] [%s]\n", priv->stub_name, priv->ext_name);
             if (hildon_file_selection_get_current_folder_iter
                 (priv->filetree, &iter)) {
                 name =
                     hildon_file_system_model_new_item(priv->model, &iter,
                                                       priv->stub_name,
                                                       priv->ext_name);
-                ULOG_INFO("Got [%s]", name);
+                DBG ("Got [%s]\n", name);
             }
         }
 
-        pos = gtk_editable_get_position(GTK_EDITABLE(priv->entry_name));
-        selection =
-          gtk_editable_get_selection_bounds(GTK_EDITABLE(priv->entry_name),
-                                            NULL, NULL);
-
         if (name)
         {
-          gboolean edited = !selection &&
-                               (pos != g_utf8_strlen (gtk_entry_get_text(GTK_ENTRY(priv->entry_name)), -1));
-          set_entry (priv->entry_name, name, priv->ext_name);
+          changed = set_entry (priv->entry_name, name, priv->ext_name);
           g_free(name);
-          if (!edited)
-            gtk_editable_select_region(GTK_EDITABLE(priv->entry_name), 0, -1);
-          else
-            /* if the user has already started to edit the name,
-               try to preserve cursor position and don't autoselect */
-            gtk_editable_set_position(GTK_EDITABLE(priv->entry_name), pos);
-        }
+		}
         else
-          set_entry (priv->entry_name, priv->stub_name, priv->ext_name);
+          changed = set_entry (priv->entry_name, priv->stub_name, priv->ext_name);
 
         g_signal_handler_unblock(priv->entry_name, priv->changed_handler);
     }
+    else
+      changed = set_entry (priv->entry_name, priv->stub_name, priv->ext_name);
+
+	if (changed) {
+    	gtk_editable_select_region(GTK_EDITABLE(priv->entry_name), 0, -1);
+		if (!gtk_widget_is_focus(GTK_WIDGET (priv->entry_name))) {
+			gtk_widget_grab_focus (GTK_WIDGET (priv->entry_name));
+		}
+	}
+
+    return;
 }
 
 /* Set PRIV->stub_name and PRIV->ext_name from NAME so that stub_name
@@ -469,12 +496,12 @@ set_stub_and_ext (HildonFileChooserDialogPrivate *priv,
   char *dot;
   gboolean is_folder;
 
-  /* XXX - We do not always reset the extension here since the old
-           code didn't do it and some code out there might rely on it
-           not being done.
-  */
+  DBG ("SET STUB AND EXT %s\n", name);
+
   g_free (priv->stub_name);
+  g_free (priv->ext_name);
   priv->stub_name = g_strdup (name);
+  priv->ext_name = NULL;
 
   /* XXX - Determine whether we are talking about a folder here.  If
            action is CREATE_FOLDER, the dialog might actually be used
@@ -679,6 +706,9 @@ check_for_location_update (GtkTreeModel *model,
   if (current_path
       && gtk_tree_path_compare (path, current_path) == 0) 
     hildon_file_chooser_dialog_update_location_info(priv);
+
+  if (current_path)
+    gtk_tree_path_free (current_path);
 }
 
 static gboolean
@@ -697,15 +727,19 @@ hildon_file_chooser_dialog_set_current_folder(GtkFileChooser * chooser,
 
     /* Now resplit the name into stub and ext parts since now the
        situation might have changed as to whether it is a folder or
-       not.
+       not.  Only do this with a non-empty stub, tho.
     */
-    if (self->priv->ext_name)
-      name = g_strconcat (self->priv->stub_name, self->priv->ext_name, NULL);
-    else
-      name = g_strdup (self->priv->stub_name);
-
-    set_stub_and_ext (self->priv, name);
-    g_free (name);
+    if (self->priv->stub_name && self->priv->stub_name[0])
+      {
+	if (self->priv->ext_name)
+	  name = g_strconcat (self->priv->stub_name,
+			      self->priv->ext_name, NULL);
+	else
+	  name = g_strdup (self->priv->stub_name);
+	
+	set_stub_and_ext (self->priv, name);
+	g_free (name);
+      }
 
     return result;
 }
@@ -978,9 +1012,17 @@ static void update_folder_button_visibility(HildonFileChooserDialogPrivate
     if (priv->should_show_folder_button
         && (priv->action == GTK_FILE_CHOOSER_ACTION_SAVE
             || priv->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER))
+      {
         gtk_widget_show(priv->folder_button);
+	g_object_set (G_OBJECT (priv->eventbox_location),
+		      "can-focus", TRUE, NULL);
+      }
     else
+      {
         gtk_widget_hide(priv->folder_button);
+	g_object_set (G_OBJECT (priv->eventbox_location),
+		      "can-focus", FALSE, NULL);
+      }
 }
 
 static void update_location_visibility(HildonFileChooserDialogPrivate *
@@ -1035,8 +1077,9 @@ static void build_ui(HildonFileChooserDialog * self)
                                    _("ckdg_fi_save_object_name"));
         }
 
-        gtk_label_set_text(GTK_LABEL(priv->label_location),
-                                     _("sfil_fi_save_objects_location"));
+
+		hildon_caption_set_label (HILDON_CAPTION (priv->caption_control_location),
+				_("sfil_fi_save_objects_location"));
         gtk_widget_hide(GTK_WIDGET(priv->filetree));
 
         /* Content pane of the filetree widget needs to be realized.
@@ -1077,8 +1120,8 @@ static void build_ui(HildonFileChooserDialog * self)
     case GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER:
         hildon_caption_set_label(HILDON_CAPTION(priv->caption_control_name),
                                  _("ckdg_fi_new_folder_name"));
-        gtk_label_set_text(GTK_LABEL(priv->label_location),
-                                     _("ckdg_fi_new_folder_location"));
+		hildon_caption_set_label (HILDON_CAPTION (priv->caption_control_location),
+						 _("ckdg_fi_new_folder_location"));
         gtk_widget_show_all(priv->caption_control_name);
         gtk_widget_hide(GTK_WIDGET(priv->filetree));
         _hildon_file_selection_realize_help(priv->filetree);
@@ -1168,6 +1211,7 @@ static GtkWidget
                                                        "file-system-model",
                                                        self->priv->model,
                                                        NULL);
+    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
     sync_current_folders(self, HILDON_FILE_CHOOSER_DIALOG(dialog));
 
     return dialog;
@@ -1191,17 +1235,10 @@ static void create_folder_callback(GtkFileSystemHandle *handle,
     if (error) {
         if (g_error_matches(error, GTK_FILE_SYSTEM_ERROR,
 			    GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS))
-            message = _("ckdg_ib_folder_already_exists");
+            message = HCS("ckdg_ib_folder_already_exists");
         else
             message = HCS("sfil_ni_operation_failed");
 
-	fprintf (stderr, "domain %d (%d)\n",
-		 error->domain, GTK_FILE_SYSTEM_ERROR);
-	fprintf (stderr, "code %d (%d)\n",
-		 error->code, GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS);
-
-        fprintf (stderr, "%s\n", error->message);
-
         hildon_banner_show_information (GTK_WIDGET(self), NULL, message);
         hildon_file_chooser_dialog_select_text(self->priv);
         gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_OK, TRUE);
@@ -1214,12 +1251,12 @@ static void create_folder_callback(GtkFileSystemHandle *handle,
     g_object_unref(self);
 }
 
-
 static void handle_folder_popup(HildonFileChooserDialog *self)
 {
   GtkFileSystem *backend;
   GtkWidget *dialog;
   gint response;
+  gchar *uri = NULL;
 
   g_return_if_fail(HILDON_IS_FILE_CHOOSER_DIALOG(self));
 
@@ -1237,8 +1274,9 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
   if (self->priv->edited)
   {
     self->priv->edited = FALSE;
-    hildon_file_chooser_dialog_set_current_name(GTK_FILE_CHOOSER(self),
-      gtk_entry_get_text(GTK_ENTRY(self->priv->entry_name)));
+    hildon_file_chooser_dialog_set_current_name 
+      (GTK_FILE_CHOOSER(self),
+       get_entry (self->priv->entry_name, self->priv->ext_name));
   }
 
   backend = _hildon_file_system_model_get_file_system(self->priv->model);
@@ -1251,37 +1289,36 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
                GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER);
 
     while ((response = gtk_dialog_run(GTK_DIALOG(dialog))) == GTK_RESPONSE_OK)
-    {
-      gchar *uri;
+      {
+	uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
+	ULOG_INFO_F("About to create folder %s", uri);
 
-      uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
-      ULOG_INFO_F("About to create folder %s", uri);
+	if (file_path)
+	  gtk_file_path_free(file_path);
 
-      if (file_path)
-        gtk_file_path_free(file_path);
+	file_path = gtk_file_system_uri_to_path(backend, uri);
 
-      file_path = gtk_file_system_uri_to_path(backend, uri);
+	/* There shouldn't be a way to invoke two simultaneous folder
+	   creating actions */
 
-      /* There shouldn't be a way to invoke two simultaneous folder
-         creating actions */
+	HildonFileChooserDialogPrivate *sub_priv =
+		HILDON_FILE_CHOOSER_DIALOG (dialog)->priv;
 
-      HildonFileChooserDialogPrivate *sub_priv =
-        HILDON_FILE_CHOOSER_DIALOG (dialog)->priv;
+	if (sub_priv->create_folder_handle)
+          {
+	    gtk_file_system_cancel_operation (sub_priv->create_folder_handle);
+	    sub_priv->create_folder_handle = NULL;
+	  }
 
-      g_assert(sub_priv->create_folder_handle == NULL);
+	/* Callback is quaranteed to be called, it unrefs the object data */
+	sub_priv->create_folder_handle =
+	  gtk_file_system_create_folder (backend, file_path,
+					 create_folder_callback,
+					 g_object_ref(dialog));
 
-      /* Callback is quaranteed to be called, it unrefs the object data */
-      sub_priv->create_folder_handle =
-        gtk_file_system_create_folder (backend, file_path,
-                                       create_folder_callback,
-                                       g_object_ref(dialog));
-
-      g_free(uri);
-
-      /* Make OK button insensitive while folder operation is going */
-      gtk_dialog_set_response_sensitive
-        (GTK_DIALOG(dialog), GTK_RESPONSE_OK, FALSE);
-    }
+	/* Make OK button insensitive while folder operation is going */
+	gtk_dialog_set_response_sensitive (GTK_DIALOG(dialog), GTK_RESPONSE_OK, FALSE);
+      }
 
     /* If user cancelled the operation, we still can have handle
      */
@@ -1292,8 +1329,11 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
     if (response == HILDON_RESPONSE_FOLDER_CREATED)
       {
         g_assert(file_path != NULL);
-        hildon_file_chooser_dialog_set_current_folder
-          (GTK_FILE_CHOOSER(self), file_path, NULL);
+        if (!hildon_file_chooser_dialog_set_current_folder
+          (GTK_FILE_CHOOSER(self), file_path, NULL) && uri) {
+			hildon_file_selection_move_cursor_to_uri (self->priv->filetree, 
+								  uri);
+		}
       }
     gtk_file_path_free(file_path);
   }
@@ -1309,6 +1349,24 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
   gtk_widget_destroy(dialog);
   gtk_window_present(GTK_WINDOW(self));
   self->priv->popup_protect = FALSE;
+  g_free(uri);
+}
+
+static gboolean
+on_location_hw_enter_pressed (GtkWidget * widget, GdkEventKey * event,
+			      gpointer data)
+{
+  HildonFileChooserDialog *self = HILDON_FILE_CHOOSER_DIALOG(data);
+  switch (event->keyval) {
+  case GDK_Return:
+    if (self->priv->should_show_folder_button
+	&& (self->priv->action == GTK_FILE_CHOOSER_ACTION_SAVE
+            || self->priv->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER))
+      handle_folder_popup(self);
+    return TRUE;
+  default:
+    return FALSE;
+  }
 }
 
 static gboolean hildon_file_chooser_dialog_location_pressed(GtkWidget
@@ -1316,8 +1374,12 @@ static gboolean hildon_file_chooser_dialog_location_pressed(GtkWidget
 {
   HildonFileChooserDialog *self = HILDON_FILE_CHOOSER_DIALOG(data);
 
-  if (self->priv->action == GTK_FILE_CHOOSER_ACTION_SAVE ||
-	self->priv->action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER)
+  fprintf (stderr, "LOCATION_PRESSED %d\n",
+	   self->priv->should_show_folder_button);
+
+  if (self->priv->should_show_folder_button
+      && (self->priv->action == GTK_FILE_CHOOSER_ACTION_SAVE
+	  || self->priv->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER))
     handle_folder_popup(self);
 
   return FALSE;
@@ -1381,13 +1443,12 @@ static void response_handler(GtkWidget * widget, gint arg1, gpointer data)
 
                     if (path_length > self->priv->max_full_path_length)
                     {
-                        GtkWidget *w;
                         g_signal_stop_emission_by_name(widget, "response");
                         hildon_file_chooser_dialog_select_text(priv);
-                        w = hildon_note_new_information(GTK_WINDOW(widget),
-                                FM("sfil_ni_target_foldername_long"));
-                        gtk_dialog_run(GTK_DIALOG(w));
-                        gtk_widget_destroy(w);
+                    	hildon_banner_show_information (widget, NULL,
+							dgettext("hildon-common-strings",
+								"file_ib_name_too_long"));
+
                     }
                     g_free(uri);
                 }
@@ -1944,20 +2005,23 @@ static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
     gtk_box_pack_start(GTK_BOX(priv->hbox_location), priv->title_location,
                        TRUE, TRUE, 0);
     eventbox = gtk_event_box_new();
+    priv->eventbox_location = eventbox;
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(eventbox), FALSE);
     gtk_container_add(GTK_CONTAINER(eventbox), priv->hbox_location);
     gtk_widget_add_events(eventbox, GDK_BUTTON_PRESS_MASK);
+    g_object_set (G_OBJECT (eventbox), "can-focus", FALSE, NULL);
+  
+    priv->caption_control_location =
+      hildon_caption_new(size_group, _("sfil_fi_save_objects_items"),
+			 eventbox, NULL,
+			 HILDON_CAPTION_OPTIONAL);
+    hildon_caption_set_separator(HILDON_CAPTION(priv->caption_control_location),
+                                 "");
 
-    priv->caption_control_location = gtk_hbox_new(FALSE, HILDON_MARGIN_DOUBLE);
-    priv->label_location = g_object_new(GTK_TYPE_LABEL, "label", _("sfil_fi_save_objects_location"), "xalign", 1.0f, NULL);
     label_items = g_object_new(GTK_TYPE_LABEL, "label",  _("sfil_fi_save_objects_items"), "xalign", 1.0f, NULL);
     priv->multiple_label = gtk_label_new(NULL);
     gtk_box_pack_start(GTK_BOX(priv->hbox_items), label_items, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(priv->hbox_items), priv->multiple_label, FALSE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(priv->caption_control_location),
-                       priv->label_location, FALSE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(priv->caption_control_location), eventbox, TRUE, TRUE, 0);
-    gtk_size_group_add_widget(size_group, priv->label_location);
     gtk_size_group_add_widget(size_group, label_items);
 
     priv->popup = gtk_menu_new();
@@ -2030,6 +2094,8 @@ static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
     g_signal_connect(eventbox, "button-release-event",
                      G_CALLBACK(hildon_file_chooser_dialog_location_pressed),
                      self);
+    g_signal_connect (eventbox, "key-press-event", 
+		      G_CALLBACK (on_location_hw_enter_pressed), self);	
 
     gtk_dialog_set_default_response(GTK_DIALOG(self), GTK_RESPONSE_OK);
 }
